(function() {

  Contextly.client.BaseRest = Contextly.createClass({

    extend: [Contextly.client.Base, Contextly.Proxy],

    /**
     *
     * @param {Contextly.client.Session} session
     * @param {Contextly.metadataProvider.Base} metadataProvider
     */
    construct: function(session, metadataProvider) {
      this.session = session;
      this.metadataProvider = metadataProvider;
    },

    getDestroyableProperties: function() {
      return [
        'session',
        'metadataProvider'
      ]
    },

    getCommonParams: function() {
      var s = Contextly.Settings;
      var m = this.metadataProvider;

      var params = {
        version: s.getKitVersion(),
        admin: s.isAdmin(),

        site_path: m.getAppId(),
        page_id: m.getPageId(),
        type: m.getPostType(),
        c_date: m.getPostCreatedDate(),
        m_date: m.getPostModifiedDate(),
        c_count: m.getPostCategoriesCount(),
        t_count: m.getPostTagsCount(),
        url: m.getPageUrl(),

        current_url: document.location.href
      };

      if (this.session.hasId()) {
        params.guid = this.session.getId();
      }

      var clientInfo = s.getClientInfo();
      if (clientInfo != null && clientInfo.client != null) {
        params.client = clientInfo.client;
        if (clientInfo.version != null) {
          params.client += ':' + clientInfo.version;
        }
      }

      return params;
    },

    /**
     * @returns {Contextly.client.Session}
     */
    getApiSession: function() {
      return this.session;
    },

    call: function(apiName, apiMethod, params, callback) {
      var url = this.getApiUrl() + apiName + '/' + apiMethod + '/';
      params = Contextly.Utils.merge(this.getCommonParams(), params);

      this.getRpc().request(
        {
          url: url,
          method: "POST",
          data: params
        },
        this.proxy(function(response) {
          if (callback == null || !response.data) {
            return;
          }

          try {
            var data = easyXDM.getJSONObject().parse(response.data);
            callback(data);
          }
          catch (e) {
            Contextly.Utils.logError('Unable to parse/handle API response', response, e);
          }
        }, false, true)
      );
    },

    getApiUrl: Contextly.abstractMethod()

  })

})();