(function($) {

  /**
   * @class
   */
  Contextly.metadataParser.ForeignHelper = Contextly.createClass( /** @lends Contextly.metadataParser.ForeignHelper.prototype */ {

    construct: function() {
      this.converters = this.getConverters();
      this.propertiesMap = this.getPropertiesMap();
    },

    getConverters: function() {
      return {};
    },

    getPropertiesMap: function() {
      return {};
    },

    /**
     * @param key
     * @param raw
     * @param [fallback]
     */
    map: function(key, raw, fallback) {
      if (typeof this.propertiesMap[key] !== 'undefined') {
        for (var i = 0; i < this.propertiesMap[key].length; i++) {
          var src = this.propertiesMap[key][i];
          var value = this.convert(src, raw[src]);
          if (typeof value !== 'undefined') {
            return value;
          }
        }
      }

      return fallback;
    },

    mapAll: function(raw, fallback) {
      var data = {};

      for (var key in this.propertiesMap) {
        var value = this.map(key, raw, fallback);
        if (typeof value !== 'undefined') {
          data[key] = value;
        }
      }

      return data;
    },

    convert: function(key, value) {
      if (value == null) {
        return;
      }

      if (this.converters[key] == null) {
        return value;
      }

      return this.converters[key](value);
    },

    convertToArray: function(value) {
      if (Contextly.Utils.isArray(value)) {
        return value;
      }
      else {
        return [value];
      }
    },

    convertToCommaSeparated: function(value) {
      if (Contextly.Utils.isArray(value)) {
        if (value.length) {
          return value.join(', ');
        }
      }
      else {
        return value;
      }
    },

    convertDateIso8601: function(value) {
      // First, split into date and time parts.
      // TODO Use Date.parse() on IE8 support dropped.
      var parts = value.match(/^([\d\-]+)(?:T([\d:]+))?/);
      if (!parts) {
        return;
      }

      // ISO 8601 allows date in both YYYY-MM-DD and YYYYMMDD formats.
      var parsedDate = parts[1].match(/^(\d+)\-(\d+)\-(\d+)$/)
        || parts[1].match(/^(\d{4})(\d{2})(\d{2})$/);
      if (!parsedDate) {
        return;
      }
      var result = parsedDate[1] + '-' + parsedDate[2] + '-' + parsedDate[3];

      // Date formats supported are: hh:mm:ss, hh:mm, hhmmss, hhmm, hh
      var parsedTime = [];
      if (parts[2]) {
        parsedTime = parts[2].match(/^(\d{2})(?::(\d{2}))?(?::(\d{2}))?$/)
          || parts[2].match(/^(\d{2})(\d{2})?(\d{2})?$/)
          || [];
      }
      var defaultTime = [null, '12', '00', '00'];
      result += ' '
        + (parsedTime[1] || defaultTime[1])
        + ':'
        + (parsedTime[2] || defaultTime[2])
        + ':'
        + (parsedTime[3] || defaultTime[3]);

      return result;
    },

    convertSplitByComma: function(value) {
      if (Contextly.Utils.isArray(value)) {
        if (value.length) {
          return value;
        }
      }
      else {
        return value.split(/\s*,\s*/);
      }
    },

    convertUrlToAbsolute: function(value) {
      if (!value) {
        return;
      }

      // Run URL through the A tag and re-construct absolute URL. We clone
      // the node, as that's the best way to get absolute URL on IE7- and
      // maybe other cases we don't know.
      // See http://stackoverflow.com/a/24437713/404521
      var a = document.createElement('a');
      a.href = value;
      a = a.cloneNode(false);
      return a.href;
    }

  });

})(jQuery);
