/**
 * @class
 *
 * @extends Contextly.Proxy
 */
Contextly.CallbacksGate = Contextly.createClass( /** @lends Contextly.CallbacksGate.prototype */{

  extend: Contextly.Proxy,

  construct: function() {
    this.left = 0;
    this.lastId = 0;
    this.sealed = true;
    this.finished = false;
    this.reasons = {};
    this.results = [];
  },

  /**
   * @returns {Function}
   */
  addReason: function() {
    var id = ++this.lastId;

    var reason = this.proxy(function() {
      this.onReasonAppeared(id);
    });

    this.reasons[id] = false;
    this.left++;

    return reason;
  },

  onReasonAppeared: function(id) {
    if (this.reasons[id]) {
      return;
    }

    this.reasons[id] = true;
    this.left--;
    this.check();
  },

  onQueueComplete: function() {
    // Block attempts to complete it twice and also set flag for isComplete().
    if (this.finished) {
      return;
    }
    this.finished = true;

    for (var i = 0; i < this.results.length; i++) {
      // In case it was destroyed by one of the results, we stop immediately.
      if (this.isDestroyed()) {
        break;
      }

      var result = this.results[i];
      var callback = result[0];
      var args = Array.prototype.slice.call(result, 1);
      callback.apply(this, args);
    }

    this.reasons = {};
    this.results = [];
  },

  /**
   * Append callback that runs as soon as all the reasons appeared.
   *
   * First argument is a callback and the rest are passed to the callback as
   * arguments.
   */
  appendResult: function(callback) {
    this.results.push(arguments);
  },

  /**
   * Prepend callback.
   *
   * @see Contextly.CallbacksGate.appendResult()
   */
  prependResult: function(callback) {
    this.results.unshift(arguments);
  },

  /**
   * Waits for the passed gate open.
   *
   * Use this function to make a gate depending on the other one.
   *
   * @param {Contextly.CallbacksGate} gate
   */
  waitFor: function(gate) {
    if (this !== gate) {
      gate.appendResult(this.addReason());
    }
  },

  check: function() {
    if (this.isSealed()) {
      return;
    }

    if (this.left === 0) {
      this.onQueueComplete();
    }
  },

  isSealed: function() {
    return this.sealed;
  },

  unseal: function() {
    this.sealed = false;
    this.check();
  },

  isComplete: function() {
    return this.finished;
  }

});
