/**
 * @class
 *
 * @extends Contextly.Proxy
 */
Contextly.CorsRequest = Contextly.createClass( /** @lends Contextly.CorsRequest.prototype */ {

  extend: Contextly.Proxy,

  /**
   * @param params
   *   Supported keys are:
   *   - url
   *   - method
   *   - success: callback to run on loading success.
   */
  construct: function(params) {
    this.params = params;
    this.response = null;
    this.sameProtocol = false;
    this.xdr = null;

    this._initXdr();
  },

  send: function() {
    var method = this.params.method || 'GET';
    var url = this.params.url;
    if (this.sameProtocol) {
      url = url.replace(/^http[s]?:\/\//, document.location.protocol + '//');
    }

    this.xdr.open(method, url);
    this.xdr.send();

    // We maintain the list of references to the request object, so that IE8
    // garbage collector don't drop the object and not abort the request.
    // See http://stackoverflow.com/a/27583212/404521
    this.id = ++Contextly.CorsRequest.lastId;
    Contextly.CorsRequest.requests[this.id] = this;
  },

  onSuccess: function() {
    this.complete = true;
    this.params.success(this.xdr.responseText);
    this.cleanup();
  },

  cleanup: function() {
    if (this.id != null && Contextly.CorsRequest.requests[this.id] != null) {
      delete Contextly.CorsRequest.requests[this.id];
    }
  },

  isComplete: function() {
    return this.complete;
  },

  _initXdr: function() {
    var xdr;

    if ('XMLHttpRequest' in window && 'withCredentials' in new XMLHttpRequest()) {
      xdr = new XMLHttpRequest();
      xdr.onreadystatechange = this.proxy(function() {
        if (xdr.readyState == 4 && xdr.status == 200) {
          this.onSuccess();
        }
      });
    }
    else if ('XDomainRequest' in window) {
      xdr = new XDomainRequest();
      xdr.onload = this.proxy(function() {
        this.onSuccess();
      });
      xdr.onerror = this.proxy(function() {
        this.cleanup();
      });

      // Hack for IE9, without it the request get aborted.
      xdr.onprogress = function() {};

      // XDomainRequest has restriction on request protocol that doesn't allow
      // us to load HTTPS assets from HTTP requests.
      this.sameProtocol = true;
    }
    else {
      Contextly.Utils.error('Unable to perform Cross-Domain request.');
    }

    this.xdr = xdr;
  }

});

Contextly.CorsRequest.lastId = 0;
Contextly.CorsRequest.requests = {};
