/**
 * Mixin with proxy features to simplify callbacks usage on rich classes.
 *
 * @class
 * @extends {Contextly.Destroyable}
 */
Contextly.Proxy = Contextly.createClass( /** @lends Contextly.Proxy.prototype */ {

  extend: Contextly.Destroyable,

  /**
   * Returns proxy function that will be called in current context.
   *
   * @param func
   * @param [passContext]
   *   If true the context of the proxy will be passed as a first argument to
   *   the callback function.
   * @param [passArguments]
   *   If true the arguments of the proxy will be passed to the callback
   *   (after) the proxy context if set.
   */
  proxy: function(func, passContext, passArguments) {
    var self = this;

    // Wrap original function to inject a switch.
    var f = function() {
      if (self.isDestroyed()) {
        return;
      }

      return func.apply(this, arguments);
    };

    // To minimize overhead of the proxy we generate 4 different functions to
    // avoid runtime checks.
    var proxy;
    if (passArguments) {
      if (passContext) {
        proxy = function() {
          Array.prototype.unshift.call(arguments, this);
          return f.apply(self, arguments);
        };
      }
      else {
        proxy = function() {
          return f.apply(self, arguments);
        };
      }
    }
    else {
      if (passContext) {
        proxy = function() {
          return f.call(self, this);
        };
      }
      else {
        proxy = function() {
          return f.call(self);
        };
      }
    }

    return proxy;
  }

});
