(function() {

  var ov = Contextly.overrides = Contextly.overrides || {};

  Contextly.Settings = Contextly.createClass({

    statics: {

      getServerUrl: function(type) {
        if (!Contextly.data.urls[type]) {
          Contextly.Utils.error(type + ' server URL not found.');
        }

        var url = null;
        var isHttps = this.isHttps();
        var urls = Contextly.data.urls[type];
        if (Contextly.Utils.isString(urls)) {
          url = urls;
        }
        else {
          var keys = {};
          keys[this.getMode()] = true;
          keys[isHttps ? 'https' : 'http'] = true;
          for (var key in keys) {
            if (urls[key] == null) {
              continue;
            }

            if (Contextly.Utils.isString(urls[key])) {
              url = urls[key];
              break;
            }
            else {
              urls = urls[key];
            }
          }
        }
        if (url == null) {
          // No URL found for the case.
          Contextly.Utils.error(type + ' server URL not found');
        }

        // URL found, add protocol if necessary. We can't use
        // protocol-relative URLs directly because easyXDM can't handle
        // them properly.
        return Contextly.Utils.resolveUrlProtocol(url, isHttps);
      },

      /**
       * Returns main Contextly server URL.
       *
       * @function
       */
      getMainServerUrl: function () {
        return ov.mainServerUrl || this.getServerUrl('main');
      },

      /**
       * Returns API server URL.
       *
       * @function
       */
      getAPIServerUrl: function () {
        return ov.apiServerUrl || this.getServerUrl('api');
      },

      /**
       * Returns Experience server URL.
       *
       * @function
       */
      getExperienceServerUrl: function () {
        return ov.experienceServerUrl || this.getServerUrl('experience');
      },

      isAdmin: function () {
        return ('admin' in ov) ? ov.admin: false;
      },

      isBrandingDisplayed: function() {
        return ('branding' in ov) ? ov.branding : true;
      },

      isHttps: function () {
        return ('https' in ov) ? ov.https : document.location.protocol === 'https:';
      },

      /**
       * Function returning information about CMS plugin.
       *
       * @returns {Object|null}
       *   Two properties are supported:
       *   - client: short name of the CMS
       *   - version: full version of the CMS plugin having format "Major.Minor"
       *   We don't pass client information in case of empty object or null
       *   returned.
       */
      getClientInfo: function() {
        return ov.client || null;
      },

      getKitVersion: function() {
        return ov.kit || Contextly.data.versions.kit;
      },

      getAssetUrl: function(path, ext) {
        var prefix = ov.assetUrl || (this.getServerUrl('cdn') + this.getKitVersion());

        // Make sure to not produce double slashes in case override has trailing
        // one already.
        return prefix.replace(/\/$/, '') + '/' + path + '.' + ext;
      },

      getMode: function () {
        return ov.mode || 'live';
      }

    }

  });

})();
