/**
 * @class
 *
 * @extends Contextly.Proxy
 */
Contextly.loader.CssManager = Contextly.createClass({

  extend: Contextly.Proxy.fn,

  statics: /** @lends Contextly.loader.CssManager */ {

    construct: function() {
      this.pending = [];

      // TODO Remove this and use querySelector() by data-ctx-key on IE8 drop.
      this.elements = {};
      this.references = {};
    },

    /**
     * @param url
     * @param key
     * @param {Contextly.CallbacksGate} gate
     */
    loadFile: function(key, url, gate) {
      var item = {
        key: key,
        url: url,
        reason: gate.addReason(),
        code: null
      };
      this.pending.push(item);

      var request = new Contextly.CorsRequest({
        url: url,
        success: this.proxy(function(text) {
          item.code = text;
          this.onFileDownloaded(item);
        }, false, true)
      });
      request.send();
    },

    /**
     * Converts relative URLs in url() statements to absolute.
     *
     * In order to support hosting Kit files on different servers we don't hard-
     * code the server URL into the aggregated CSS files and replace it
     * on-the-fly instead.
     */
    convertUrlsToAbsolute: function(fileUrl, code) {
      var baseUrl = fileUrl.replace(/[^\/]+$/, '');
      return code.replace(/(url\(\s*['"]?)([^"'\)]+)/gi, function(match, prefix, url) {
        if (url.match(/^(\?|#|\/|http[s]?:\/\/)/)) {
          // Absolute or having no paths go untouched.
          return match;
        }
        else {
          return prefix + baseUrl + url;
        }
      });
    },

    onFileDownloaded: function(affected) {
      affected.code = this.convertUrlsToAbsolute(affected.url, affected.code);

      while (this.pending[0] && this.pending[0].code !== null) {
        var item = this.pending.shift();

        var style = this.buildStyleElement(item.key, item.code);
        Contextly.Utils.addHeadElement(style);

        setTimeout(item.reason, 0);
      }
    },

    buildStyleElement: function(key, code) {
      var style = document.createElement('style');
      style.type = 'text/css';
      style.setAttribute('data-ctx-key', key);

      if (style.styleSheet) {
        style.styleSheet.cssText = code;
      }
      else {
        style.appendChild(document.createTextNode(code));
      }

      this.elements[key] = style;

      return style;
    },

    /**
     * @param key
     * @param {Function|string} code
     *   In case function is passed, it will only be called on
     */
    loadCode: function(key, code) {
      if (this.styleExists(key)) {
        this.references[key]++;
        return;
      }

      if (Contextly.Utils.isFunction(code)) {
        code = code();

        if (code == null || code === '') {
          return;
        }
      }

      this.references[key] = this.references[key] || 0;
      this.references[key]++;

      var style = this.buildStyleElement(key, code);
      Contextly.Utils.addHeadElement(style);
    },

    styleExists: function(key) {
      return !!this.elements[key];
    },

    removeCode: function(key) {
      if (!this.styleExists(key)) {
        return;
      }

      this.references[key]--;
      if (this.references[key] > 0) {
        return;
      }

      if (this.elements[key].parentNode) {
        this.elements[key].parentNode.removeChild(this.elements[key]);
      }
      delete this.elements[key];
      delete this.references[key];
    }

  }

});