Contextly.loader.JsCorsLoader = Contextly.createClass({

  extend: Contextly.Proxy,

  construct: function() {
    this.pending = [];
  },

  load: function(key, url, gate) {
    var item = {
      key: key,
      url: url,
      reason: gate.addReason(),
      code: null
    };
    this.pending.push(item);

    var request = new Contextly.CorsRequest({
      url: url,
      success: this.proxy(function(text) {
        item.code = text;
        this.onFileDownloaded(item);
      }, false, true)
    });
    request.send();
  },

  onFileDownloaded: function(affected) {
    while (this.pending[0] && this.pending[0].code !== null) {
      var item = this.pending.shift();
      item.code += "\n//# sourceURL=" + item.url;
      this.execScript(item.code);
      item.reason();
    }
  },

  execScript: function(data) {
    // jQuery.globalEval copy.
    (
      window.execScript ||
      function (data) {
        window["eval"].call(window, data);
      }
    )(data);
  }

});

/**
 * @class
 *
 * @extends Contextly.Proxy
 */
Contextly.loader.JsIeLoader = Contextly.createClass( /** @lends Contextly.loader.JsIeLoader.prototype */{

  extend: Contextly.Proxy,

  construct: function() {
    this.loading = [];
    this.loaded = [];
  },

  onStateChange: function() {
    var script;

    while (this.loading[0] && this.loading[0].readyState == 'loaded') {
      script = this.loading.shift();

      // Put into another queue to wait for "complete" event, since IE8 doesn't
      // fire "onload" event.
      this.loaded.push(script);

      // Can't just appendChild, old IE bug if element isn't closed.
      Contextly.Utils.addHeadElement(script);
    }

    while (this.loaded[0] && this.loaded[0].readyState == 'complete') {
      script = this.loaded.shift();

      // Avoid future events from this script (eg, if src changes).
      script.onreadystatechange = null;

      // Notify gates about completion.
      script.ctxReason();
    }
  },

  load: function(key, url, gate) {
    var script = document.createElement('script');
    script.setAttribute('data-ctx-key', key);
    script.ctxReason = gate.addReason();
    this.loading.push(script);

    script.onreadystatechange = this.proxy(this.onStateChange);

    // Must set src AFTER adding onreadystatechange listener,
    // otherwise we would miss the loaded event for cached scripts.
    script.src = url;
  }

});

Contextly.loader.JsManager = Contextly.createClass({

  statics: /** @lends Contextly.loader.JsManager */ {

    construct: function() {
      this.loader = this._detectLoader();
    },

    _detectLoader: function() {
      var script = document.createElement('script');
      if (!('async' in script) && ('readyState' in script)) {
        // Prefer native way on IE 7-9.
        return new Contextly.loader.JsIeLoader();
      }
      else {
        // Since Firefox has bug on 64bit Windows and Linux (at least 45.0.1)
        // with maintaining the execution order of scripts inserted with
        // async=false, we have to use CORS loader even for modern browsers.
        return new Contextly.loader.JsCorsLoader();
      }
    },

    loadFile: function(key, url, gate) {
      this.loader.load(key, url, gate);
    }

  }

});

