Contextly.assets = Contextly.assets || {};

Contextly.loader.PackageManager = Contextly.createClass({

  extend: [Contextly.Transmitter.fn, Contextly.Proxy.fn],

  statics: {

    construct: function() {
      this.packages = {};
      this.loading = {};
      this.callbacks = {};
      this.options = {
        js: {},
        css: {}
      };
    },

    getConfig: function(name) {
      if (!Contextly.assets[name]) {
        throw 'Package assets not found: ' + name;
      }

      return Contextly.assets[name];
    },

    setOptions: function(options) {
      this.options = Contextly.Utils.merge(this.options, options);
    },

    addCallback: function(name, callback) {
      this.callbacks[name] = this.callbacks[name] || [];
      this.callbacks[name].push(callback);
    },

    /**
     * @param {String} name
     * @param {Contextly.CallbacksGate} gate
     */
    load: function(name, gate) {
      gate = gate || new Contextly.loader.CallbacksGate();

      this.loadPackage(name, gate);

      return gate;
    },

    /**
     * @param {String} name
     * @param {Contextly.CallbacksGate} gate
     */
    loadPackage: function(name, gate) {
      if (this.isPackageExcluded(name)) {
        return;
      }

      // Don't try to load packages that are already in progress or loaded.
      if (name in this.packages) {
        // In case package loading is in progress just queue current gate to
        // open right after previous.
        if (this.packages[name] === Contextly.loader.packageStates.LOADING) {
          gate.waitFor(this.loading[name]);
        }

        return;
      }

      var config = this.getConfig(name);
      this.loading[name] = gate;
      this.packages[name] = Contextly.loader.packageStates.LOADING;
      this.broadcast(Contextly.loader.broadcastTypes.LOADING, name, config, gate);

      // Prepend registered callbacks in reverse order to keep the right order,
      // then internal handler to do the cleanup right before
      if (name in this.callbacks) {
        for (var i = this.callbacks[name].length - 1; i >= 0; i--) {
          gate.prependResult(this.callbacks[name][i]);
        }
      }
      gate.prependResult(this.proxy(this.onPackageLoaded, false, true), name);

      this.loadDependencies(config, gate);
      this.loadAssets(config, gate);
      this.loadTemplates(config, gate);
    },

    loadDependencies: function(config, gate) {
      if (!config.dependencies || !config.dependencies.length) {
        return;
      }

      for (var i = 0; i < config.dependencies.length; i++) {
        this.loadPackage(config.dependencies[i], gate);
      }
    },

    loadAssets: function(config, gate) {
      var i, path, url, manager;

      var types = {
        js: Contextly.loader.JsManager,
        css: Contextly.loader.CssManager
      };
      for (var type in types) {
        if (!(type in config)) {
          continue;
        }

        if (this.isExcluded(this.options[type], '*')) {
          continue;
        }

        manager = types[type];
        for (i = 0; i < config[type].length; i++) {
          path = config[type][i];
          if (this.isExcluded(this.options[type], path)) {
            continue;
          }

          if (config.foreign) {
            url = path;
          }
          else {
            url = Contextly.Settings.getAssetUrl(path, type);
          }

          manager.loadFile(path, url, gate);
        }
      }
    },

    loadTemplates: function(config, gate) {
      if (config.tpl) {
        Contextly.loader.TemplatesManager.load(config, gate);
      }
    },

    isPackageExcluded: function(name) {
      var libs = Contextly.loader.LibrariesManager.getFlags();
      return this.isExcluded(libs, "*") || this.isExcluded(libs, name);
    },

    isExcluded: function(list, name) {
      return (name in list && !list[name]);
    },

    onPackageLoaded: function(name) {
      if (name in this.loading) {
        delete this.loading[name];
      }

      var config = this.getConfig(name);
      if ('expose' in config) {
        this.exposePackages(config.expose);
        delete config.expose;
      }

      this.packages[name] = Contextly.loader.packageStates.LOADED;
      this.broadcast(Contextly.loader.broadcastTypes.LOADED, name);
    },

    exposePackages: function(packages) {
      Contextly.assets = Contextly.Utils.merge(Contextly.assets, packages);
    }

  }

});

Contextly.loader.packageStates = {
  LOADING: 'loading',
  LOADED: 'loaded'
};

Contextly.loader.broadcastTypes = {
  LOADING: 'contextlyAssetsLoading',
  LOADED: 'contextlyAssetsLoaded'
};
