Contextly.templates = Contextly.templates || {};

Contextly.loader.TemplatesManager = Contextly.createClass({

  extend: Contextly.Proxy.prototype,

  statics: /** @lends Contextly.loader.TemplatesManager */{

    construct: function() {
      this.states = {};
      this.callbacks = {};
      this.files = {};
    },

    /**
     * Loads templates.
     *
     * The process is quite complicated and requires description:
     * - First we load all the template files, but don't compile them, because
     *   Handlebars compiler is probably a part of the package with templates
     *   and it may still be pending.
     * - In case the template is loading or loaded, we just add gate's reason to
     *   the list of callbacks for this template.
     * - In case the template is already compiled we do nothing.
     * - On package loaded we compile all the loaded templates.
     *
     * @param config
     * @param {Contextly.CallbacksGate} gate
     */
    load: function(config, gate) {
      for (var name in config.tpl) {
        var path = config.tpl[name];
        var url = Contextly.Settings.getAssetUrl(path, 'handlebars');
        this.loadFile(name, url, gate);
      }

      gate.prependResult(this.proxy(this.onPackageLoaded));
    },

    /**
     *
     * @param name
     * @param url
     * @param {Contextly.CallbacksGate} gate
     */
    loadFile: function(name, url, gate) {
      if (this.states[name]) {
        switch (this.states[name]) {
          case Contextly.loader.templateStates.COMPILED:
            return;

          case Contextly.loader.templateStates.LOADED:
          case Contextly.loader.templateStates.LOADING:
            this.callbacks[name].push(gate.addReason());
            return;
        }
      }

      this.states[name] = Contextly.loader.templateStates.LOADING;
      this.callbacks[name] = this.callbacks[name] || [];
      this.callbacks[name].push(gate.addReason());

      var request = new Contextly.CorsRequest({
        url: url,
        success: this.proxy(function(contents) {
          this.onFileDownloaded(name, contents);
        }, false, true)
      });
      request.send();
    },

    onFileDownloaded: function(name, contents) {
      this.files[name] = contents;
      this.states[name] = Contextly.loader.templateStates.LOADED;

      // Run callbacks, so that package gate could open. We compile templates
      // on package loaded with the gate result added in load().
      for (var i = 0; i < this.callbacks[name].length; i++) {
        this.callbacks[name][i]();
      }
      delete this.callbacks[name];
    },

    onPackageLoaded: function() {
      for (var name in this.files) {
        this.states[name] = Contextly.loader.templateStates.LOADED;
        Contextly.templates[name] = Handlebars.compile(this.files[name]);
      }

      this.files = {};
    }

  }

});

Contextly.loader.templateStates = {
  LOADING: 'loading',
  LOADED: 'loaded',
  COMPILED: 'compiled'
};
