(function($) {

  /**
   * Widget container mixin providing auto-placement.
   *
   * @mixin
   */
  Contextly.widget.AutoPlacement = Contextly.createClass({

    isAutoPlacementEnabled: function() {
      if (!this.hasData()) {
        return false;
      }

      var data = this.getData();
      return data.settings.auto_placement && data.settings.auto_placement_enabled;
    },

    hasBeenAutoPlaced: function() {
      return !!this.autoPlaced;
    },

    getAutoPlacementDefaults: function() {
      return {
        article: '',
        paragraph: '> p',
        place: 'before',
        index: 3,
        from: 'start',
        stop: '.ctx-no-auto-placement',
        threshold: 0
      };
    },

    getAutoPlacementSettings: function() {
      return $.extend(this.getAutoPlacementDefaults(), this.getData().settings.auto_placement);
    },

    findArticleContainer: function(s) {
      var sel = {
        article: '.ctx-article:first',
        hint: '.ctx-article-root:first',
        custom: s.article
      };

      var $context = this.getContext();

      if ($context.is(sel.article)) {
        return $context;
      }

      var $candidate = $context.find(sel.article);
      if ($candidate.length) {
        return $candidate;
      }

      var $hint = $context.find(sel.hint);
      if ($hint.length) {
        return $hint.parent();
      }

      if (sel.custom) {
        if ($context.is(sel.custom)) {
          return $context;
        }

        $candidate = $context.find(sel.custom)
          .filter(':eq(0)');
        if ($candidate.length) {
          return $candidate;
        }
      }

      return null;
    },

    autoPlaceElements: function() {
      this.autoPlaced = true;

      var s = this.getAutoPlacementSettings();
      var sel = {
        stop: s.stop,
        paragraph: s.paragraph || 'p'
      };

      var $article = this.findArticleContainer(s);
      if (!$article || !$article.length || $article.is(sel.stop)) {
        return null;
      }

      // Collect all non-empty paragraphs, for performance reasons in two steps
      // See https://api.jquery.com/parent-selector/
      var $paragraphs = $article.find(sel.paragraph)
        .filter(':parent')
        .filter(function() {
          // Only take non-empty paragraphs into account.
          return !!Contextly.Utils.trim($(this).text());
        });
      if (!$paragraphs.length || $paragraphs.length < s.threshold) {
        return null;
      }

      var $target, index;
      if (s.from === 'start') {
        index = Math.min($paragraphs.length - 1, s.index - 1);
        $target = $paragraphs.eq(index);
      }
      else {
        // Negative index is supported since 1.4 and we support older versions.
        index = Math.max($paragraphs.length - s.index, 0);
        $target = $paragraphs.eq(index);
      }
      if (!$target.length) {
        return null;
      }

      var $container = $('<div />')
        .attr({
          "class": this.getTypeClass()
        });
      if (s.place === 'before') {
        $target.before($container);
      }
      else {
        $target.after($container);
      }

      return $container;
    }

  });

})(jQuery);