(function($) {
  /**
   * @class
   */
  Contextly.widget.BaseContainers = Contextly.createClass({

    /**
     * @param $context
     * @param [data]
     */
    construct: function($context, data) {
      this.data = data;
      this.$context = $context;
    },

    getElements: function() {
      if (this.elements == null) {
        this.elements = this.findElements();
      }

      return this.elements;
    },

    findElements: function() {
      return $('.' + this.getClass(), this.$context);
    },

    getTypeClass: Contextly.abstractMethod(),

    getClass: function() {
      return this.getTypeClass();
    },

    hasData: function() {
      return !!this.data;
    },

    getData: function() {
      if (!this.hasData()) {
        Contextly.Utils.error('Widget data is not set on the container.');
      }

      return this.data;
    },

    getContext: function() {
      return this.$context;
    },

    getWidgetPackageName: function() {
      // Return nothing for now.
    },

    getStylistPackageName: function() {
      // Return nothing for now.
    },

    getThemePackageName: function() {
      // Return nothing for now.
    }

  });


  /**
   * @class
   * @extends Contextly.widget.BaseContainers
   */
  Contextly.widget.SnippetContainers = Contextly.createClass({

    extend: Contextly.widget.BaseContainers,

    getTypeClass: function() {
      return 'ctx-module-container';
    },

    getThemePackageName: function() {
      return 'widgets/snippet/' + this.getData().settings.display_type;
    }

  });

  /**
   * @class
   * @extends Contextly.widget.BaseContainers
   */
  Contextly.widget.SidebarContainers = Contextly.createClass({

    extend: Contextly.widget.BaseContainers,

    getTypeClass: function() {
      return 'ctx-sidebar-container';
    },

    getClass: function() {
      if (this.data && this.data.id != null) {
        return 'ctx-sidebar-container--' + this.data.id;
      }
      else {
        return this.getTypeClass();
      }
    },

    getThemePackageName: function() {
      return 'widgets/sidebar/default';
    }

  });

  /**
   * @class
   * @extends Contextly.widget.BaseContainers
   * @extends Contextly.widget.AutoPlacement
   */
  Contextly.widget.AutoSidebarContainers = Contextly.createClass({

    extend: [Contextly.widget.BaseContainers, Contextly.widget.AutoPlacement],

    construct: function() {
      this.autoPlaced = false;

      return Contextly.widget.BaseContainers.apply(this, arguments);
    },

    getTypeClass: function() {
      return 'ctx-autosidebar-container';
    },

    getClass: function() {
      if (this.data && this.data.id != null) {
        return 'ctx-autosidebar-container--' + this.data.id;
      }
      else {
        return this.getTypeClass();
      }
    },

    getThemePackageName: function() {
      return 'widgets/sidebar/default';
    },

    getElements: function() {
      var $elements = Contextly.widget.BaseContainers.fn.getElements.apply(this, arguments);

      // In case no elements have been selected, try auto-placement.
      if (!$elements.length && this.isAutoPlacementEnabled() && !this.hasBeenAutoPlaced()) {
        var $autoplaced = this.autoPlaceElements();
        if ($autoplaced && $autoplaced.length) {
          $elements = this.elements = $autoplaced;
        }
      }

      return $elements;
    },

    getAutoPlacementDefaults: function() {
      var defs = Contextly.widget.AutoPlacement.fn.getAutoPlacementDefaults.apply(this, arguments);

      defs.stop = '.ctx-no-autosidebar';

      return defs;
    }

  });

  /**
   * @class
   * @extends Contextly.widget.BaseContainers
   */
  Contextly.widget.StoryLineSubscribeContainers = Contextly.createClass({

    extend: Contextly.widget.BaseContainers,

    getTypeClass: function() {
      return 'ctx-subscribe-container';
    },

    getThemePackageName: function() {
      return 'widgets/storyline/' + this.getData().settings.theme;
    }

  });

  /**
   * @class
   * @extends Contextly.widget.BaseContainers
   */
  Contextly.widget.PersonalizationSubscribeContainers = Contextly.createClass({

    extend: Contextly.widget.BaseContainers,

    getTypeClass: function() {
      return 'ctx-personalization-container';
    },

    getThemePackageName: function() {
      return 'widgets/personalization/' + this.getData().settings.theme;
    }

  });

  /**
   * @class
   * @extends Contextly.widget.BaseContainers
   * @extends Contextly.widget.AutoPlacement
   */
  Contextly.widget.ChannelSubscribeContainers = Contextly.createClass({

    extend: [Contextly.widget.BaseContainers, Contextly.widget.AutoPlacement],

    getTypeClass: function() {
      return 'ctx-channel-container';
    },

    getThemePackageName: function() {
      return 'widgets/channel/' + this.getData().settings.theme;
    },

    getElements: function() {
      var $elements = Contextly.widget.BaseContainers.fn.getElements.apply(this, arguments);

      // In case no elements have been selected, try auto-placement.
      if (!$elements.length && this.isAutoPlacementEnabled() && !this.hasBeenAutoPlaced()) {
        var $autoplaced = this.autoPlaceElements();
        if ($autoplaced && $autoplaced.length) {
          $elements = this.elements = $autoplaced;
        }
      }

      return $elements;
    },

    getAutoPlacementDefaults: function() {
      var defs = Contextly.widget.AutoPlacement.fn.getAutoPlacementDefaults.apply(this, arguments);

      defs.stop = '.ctx-no-channel-subscribe';

      return defs;
    }

  });

  /**
   * @class
   * @extends Contextly.widget.BaseContainers
   */
  Contextly.widget.SiderailContainers = Contextly.createClass({

    extend: Contextly.widget.BaseContainers,

    getTypeClass: function() {
      return 'ctx-siderail-container';
    },

    getThemePackageName: function() {
      return 'widgets/siderail/' + this.getData().settings.theme;
    }

  });

  /**
   * @class
   * @extends Contextly.widget.BaseContainers
   */
  Contextly.widget.SocialContainers = Contextly.createClass({

    extend: Contextly.widget.BaseContainers,

    getTypeClass: function() {
      return 'ctx-social-container';
    },

    getThemePackageName: function() {
      return 'widgets/social/carousel';
    }

  });

})(jQuery);