// Init global namespace.
Contextly.widget = Contextly.widget || {};

// Widget types.
Contextly.widget.types = {
  SNIPPET: 'snippet',
  SIDEBAR: 'sidebar',
  AUTO_SIDEBAR: 'auto-sidebar',
  STORYLINE_SUBSCRIBE: 'storyline-subscribe',
  SIDERAIL: 'side-rail',
  SOCIAL: 'social',
  PERSONALIZATION_SUBSCRIBE: 'personalization-subscribe',
  CHANNEL_SUBSCRIBE: 'channel-subscribe'
};

// Snippet styles.
Contextly.widget.styles = {
  TEXT: 'default',
  TABS: 'tabs',
  BLOCKS: 'blocks',
  BLOCKS2: 'blocks2',
  FLOAT: 'float'
};

// SideRail styles.
Contextly.widget.sideRailStyles = {
  DEFAULT: 'default',
  H_SQUARE: 'h-square',
  H_LETTER: 'h-letter'
};

// Widget link types.
Contextly.widget.linkTypes = {
  PREVIOUS: 'previous',
  RECENT: 'recent',
  WEB: 'web',
  INTERESTING: 'interesting',
  CUSTOM: 'custom',
  PROMO: 'sticky'
};

// Widget recommendation types.
Contextly.widget.recommendationTypes = {
  VIDEO: 10,
  PRODUCT: 9,
  COOKIE: 8,
  EVERGREEN: 7,
  OPTIMIZATION: 6
};

// Widget-related broadcast event types.
Contextly.widget.broadcastTypes = {
  DISPLAYED: 'contextlyWidgetDisplayed',
  REMOVED: 'contextlyWidgetRemoved',
  IN_VIEWPORT: 'contextlyWidgetInViewport',
  LAYOUT_CHANGED: 'contextlyWidgetLayoutChanged'
};

// Widget-related API event names.
Contextly.widget.eventNames = {
  MODULE_VIEW: 'module_view',
  CAROUSEL_NAVIGATION: 'carousel_navigation'
};

// Unique index of the widget.
Contextly.widget.index = 0;

/**
 * @class
 */
Contextly.widget.Factory = Contextly.createClass({ /** @lends Contextly.widget.Factory.prototype */

  construct: function($context) {
    this.$context = $context;
  },

  getContainerClass: function(type) {
    switch (type) {
      case Contextly.widget.types.SIDEBAR:
        return Contextly.widget.SidebarContainers;

      case Contextly.widget.types.AUTO_SIDEBAR:
        return Contextly.widget.AutoSidebarContainers;

      case Contextly.widget.types.STORYLINE_SUBSCRIBE:
        return Contextly.widget.StoryLineSubscribeContainers;

      case Contextly.widget.types.PERSONALIZATION_SUBSCRIBE:
        return Contextly.widget.PersonalizationSubscribeContainers;

      case Contextly.widget.types.CHANNEL_SUBSCRIBE:
        return Contextly.widget.ChannelSubscribeContainers;

      case Contextly.widget.types.SIDERAIL:
        return Contextly.widget.SiderailContainers;

      case Contextly.widget.types.SOCIAL:
        return Contextly.widget.SocialContainers;

      default:
        return Contextly.widget.SnippetContainers;
    }
  },

  /**
   * @param type
   * @param [data]
   */
  getContainer: function(type, data) {
    var containerClass = this.getContainerClass(type);
    if (!containerClass) {
      return null;
    }

    return new containerClass(this.$context, data);
  },

  getWidgetClass: function(type, theme) {
    switch (type) {
      case Contextly.widget.types.SIDEBAR:
        return Contextly.widget.Sidebar;

      case Contextly.widget.types.AUTO_SIDEBAR:
        return Contextly.widget.AutoSidebar;

      case Contextly.widget.types.STORYLINE_SUBSCRIBE:
        return Contextly.widget.StoryLineSubscribe;

      case Contextly.widget.types.PERSONALIZATION_SUBSCRIBE:
        return Contextly.widget.PersonalizationSubscribe;

      case Contextly.widget.types.CHANNEL_SUBSCRIBE:
        return Contextly.widget.ChannelSubscribe;

      case Contextly.widget.types.SIDERAIL:
        switch (theme) {
          case Contextly.widget.sideRailStyles.H_SQUARE:
            return Contextly.widget.SiderailHSquare;

          case Contextly.widget.sideRailStyles.H_LETTER:
            return Contextly.widget.SiderailHLetter;

          default:
            return Contextly.widget.Siderail;
        }

      case Contextly.widget.types.SOCIAL:
        return Contextly.widget.Social;

      default:
        // Snippet.
        switch (theme) {
          case Contextly.widget.styles.TEXT:
            return Contextly.widget.TextSnippet;

          case Contextly.widget.styles.TABS:
          case Contextly.widget.styles.BLOCKS:
            return Contextly.widget.BlocksSnippet;

          case Contextly.widget.styles.BLOCKS2:
            return Contextly.widget.Blocks2Snippet;

          case Contextly.widget.styles.FLOAT:
            return Contextly.widget.FloatSnippet;
        }
    }
  },

  /**
   * @param params
   *
   * @returns {Contextly.widget.Base|null}
   */
  getWidget: function(params) {
    var required = {
      data: true,
      container: true,
      eventsLogger: true,
      experienceClient: true,
      mainServerClient: true
    };
    for (var key in required) {
      if (params[key] == null) {
        Contextly.Utils.error('Required parameter ' + key + ' not found.');
      }
    }

    var data = params.data;
    var widgetTheme = data.settings.display_type || data.settings.theme;
    var widgetClass = this.getWidgetClass(data.type, widgetTheme);
    if (!widgetClass) {
      return null;
    }

    return new widgetClass(params);
  }

});
