(function($) {

  /**
   * @class
   * @extends Contextly.overlay.Base
   */
  Contextly.overlay.PersonalizationSubscribe = Contextly.createClass({

    extend: Contextly.overlay.Base,

    statics: /** @lends Contextly.overlay.PersonalizationSubscribe */ {

      open: function(options) {
        Contextly.overlay.Base.open.call(this, null, options);
      },

      renderOverlay: function() {
        var init = Contextly.overlay.Base.renderOverlay.apply(this, arguments);
        if (init) {
          this.overlay.addClass('ctx-personalization-overlay');
        }
      },

      hasLinks: function() {
        return !!this.options.links.length;
      },

      isSubscriptionEnabled: function() {
        return this.options.subscription;
      },

      isSubscribed: function() {
        return this.options.subscribed;
      },

      isConfirmed: function() {
        return this.options.confirmed;
      },

      getHeaderTitle: function() {
        return this.dialog.find('.ctx-header-title');
      },

      getSubscriptionSection: function() {
        return this.dialog.find('.ctx-subscription-section');
      },

      getStoriesSection: function() {
        return this.dialog.find('.ctx-stories-section');
      },

      getMessageRow: function() {
        return this.dialog.find('.ctx-message-row');
      },

      getEmailInput: function() {
        return this.dialog.find('.ctx-email');
      },

      getSubmit: function() {
        return this.dialog.find('.ctx-submit');
      },

      getEmailRow: function() {
        return this.dialog.find('.ctx-email-row');
      },

      getCloseLink: function() {
        return this.dialog.find('.ctx-dialog-close');
      },

      renderDialog: function() {
        if (!this.dialog) {
          var classes = [ 'ctx-personalization-dialog', 'ctx-overlay-dialog' ];
          if (!Contextly.Browser.hasSvg()) {
            classes.push('ctx-no-svg');
          }
          if (this.hasLinks()) {
            classes.push('ctx-with-stories');
          }
          if (this.isSubscriptionEnabled()) {
            classes.push('ctx-with-subscription');
          }

          var content =
            '<div class="' + Contextly.Utils.escape(classes.join(' ')) + '">'
            + '<div class="ctx-header">'
              + '<a href="javascript:" class="ctx-dialog-close">X</a>'
              + '<span class="ctx-header-title"></span>'
            + '</div>'
            + '<div class="ctx-body">'
              + '<div class="ctx-message-row"></div>'
              + '<div class="ctx-subscription-section"></div>'
              + '<div class="ctx-stories-section"></div>'
            + '</div>'
            + '<div class="ctx-footer ctx-clearfix">'
              /* TODO NOW + '<a class="ctx-how-link" target="_blank" href="http://contextly.com/">How does this work?</a>'*/
              + '<a class="ctx-logo-link" target="_blank" href="http://contextly.com">Powered by</a>'
            + '</div>'
            + '</div>';
          this.dialog = $(content)
            .appendTo('body');
        }
        else {
          // Show dialog.
          this.dialog
            .show();
        }

        if (this.isSubscriptionEnabled() && !this.isSubscribed()) {
          this.getHeaderTitle()
            .text('Subscribe to Your Personalized Stories');
        }
        else {
          this.getHeaderTitle()
            .text('Your Personalized Stories');
        }

        if (this.isSubscriptionEnabled()) {
          if (this.isSubscribed()) {
            if (this.isConfirmed()) {
              this.successMessage("You are already subscribed.", !this.hasLinks());
            }
            else {
              this.successMessage("Almost done! Check your email.", !this.hasLinks());
            }
          }
          else {
            content =
              '<div class="ctx-email-row">'
                + '<div class="ctx-email-wrp">'
                + '<input type="email" class="ctx-email" placeholder="Email address" />'
                + '</div>'
                + '<div class="ctx-submit-wrp">'
                + '<button class="ctx-submit">'
                + '<div class="ctx-submit-label">Subscribe</div>'
                + '<div class="ctx-icon ctx-icon-hourglass"></div>'
                + '</button>'
                + '</div>'
                + '</div>';
            this.getSubscriptionSection()
              .html(content)
              .show();
          }
        }

        if (this.hasLinks()) {
          content = '';
          this.each(this.options.links, function(link) {
            var post = '';
            post += '<h3 class="ctx-post-title">'
              + '<a class="ctx-post-title-link"'
                + ' href="' + Contextly.Utils.escape(link.native_url) + '"'
                + this.getEventTrackingHtml(link)
                + '>'
              + link.title
              + '</a>'
              + '</h3>';

            if (link.thumbnail_url || link.description) {
              var post_classes = ['ctx-post-content', 'ctx-clearfix'];
              if (link.thumbnail_url) {
                post_classes.push('ctx-with-img');
              }
              if (link.description) {
                post_classes.push('ctx-with-summary');
              }

              post += '<div class="' + Contextly.Utils.escape(post_classes.join(' ')) + '">';

              if (link.thumbnail_url) {
                post += '<a class="ctx-post-img-link"'
                    + ' href="' + Contextly.Utils.escape(link.native_url) + '"'
                    + this.getEventTrackingHtml(link)
                    + '>'
                  + '<img class="ctx-post-img" alt="" src="' + Contextly.Utils.escape(link.thumbnail_url) + '" />'
                  + '</a>';
              }

              if (link.description) {
                post += '<div class="ctx-post-summary">'
                  + '<a class="ctx-post-summary-link"'
                    + ' href="' + Contextly.Utils.escape(link.native_url) + '"'
                    + this.getEventTrackingHtml(link)
                    + '>'
                  + link.description
                  + '</a>'
                  + '</div>';
              }

              post += '</div>';
            }

            content += '<div class="ctx-post">'
              + post
              + '</div>';
          });

          this.getStoriesSection()
            .html(content)
            .show();
        }
      },

      destroyDialog: function() {
        this.dialog.hide();

        this.getMessageRow()
          .empty()
          .hide();
        this.getStoriesSection()
          .empty()
          .hide();
        this.getSubscriptionSection()
          .empty()
          .hide();
      },

      bindHandlers: function() {
        Contextly.overlay.Base.bindHandlers.apply(this, arguments);

        this.getCloseLink()
          .bind(this.ns('click'), this.proxy(this.close));

        if (this.isSubscriptionEnabled()) {
          this.getSubmit()
            .bind(this.ns('click'), this.proxy(this.onSubmit, false, true));
          this.getEmailInput()
            .bind(this.ns('keypress'), this.proxy(this.onEmailInputKeyPress, false, true));
        }
      },

      unbindHandlers: function() {
        Contextly.overlay.Base.unbindHandlers.apply(this, arguments);

        this.getCloseLink()
          .unbind(this.ns('click'));

        if (this.isSubscriptionEnabled()) {
          this.getSubmit()
            .unbind(this.ns('click'));
          this.getEmailInput()
            .unbind(this.ns('keypress'));
        }
      },

      onEmailInputKeyPress: function(e) {
        if (e.which == 13) {
          this.onSubmit(e);
        }
      },

      onSubmit: function(e) {
        e.preventDefault();

        var params = {};
        var email = $.trim(this.getEmailInput().val());
        if (!email || !Contextly.Utils.isEmailValid(email)) {
          this.errorMessage('Please enter valid email address.');
          return;
        }

        params.email = email;
        this.subscribeToPersonalization(params);
      },

      getEventTrackingHtml: function(link) {
        return ' onmousedown="this.href=' + Contextly.Utils.escape(JSON.stringify(link.url)) + '"';
      },

      subscribeToPersonalization: function(params) {
        this.lockSubscriptionUI();
        this.hideMessage();

        params = params || {};
        var callback = this.proxy(this.onSubscribeComplete, false, true);
        this.options.experienceClient.call('personalization', 'subscribe-email', params, callback);
      },

      onSubscribeComplete: function(response) {
        if (response && response.success) {
          if (response.entry.confirmed) {
            this.successMessage("You've been successfully subscribed.", true);
          }
          else {
            this.successMessage('Almost done! Check your email.', true);
          }

          this.getSubscriptionSection()
            .hide();

          this.broadcastPersonalizationStatus({
            subscribed: true,
            confirmed: response.entry.confirmed
          });
        }
        else {
          var message = 'Something went wrong.';
          if (response && response.message) {
            message = response.message;
          }
          this.errorMessage(message);
          this.unlockSubscriptionUI();
          this.focusEmailInput();
        }
      },

      lockSubscriptionUI: function() {
        this.getEmailInput()
          .attr('disabled', 'disabled');
        this.getEmailRow()
          .addClass('ctx-progress');
        this.getSubmit()
          .attr('disabled', 'disabled')
          .not('.ctx-progress')
          .addClass('ctx-progress');
      },

      unlockSubscriptionUI: function() {
        this.getEmailInput()
          .removeAttr('disabled');
        this.getEmailRow()
          .removeClass('ctx-progress');
        this.getSubmit()
          .removeAttr('disabled')
          .removeClass('ctx-progress');
      },

      focusEmailInput: function() {
        this.getEmailInput()
          .focus();
      },

      showMessage: function(text, className) {
        var messageRow = this.getMessageRow();
        var duration = this.options.duration;

        var fadeIn = this.proxy(function() {
          messageRow
            .removeClass('ctx-success ctx-error ctx-info')
            .addClass(className)
            .text(text)
            .animate({opacity: 1}, duration);
        });

        if (messageRow.is(':visible')) {
          duration /= 2;
          messageRow
            .stop(true)
            .animate({opacity: 0}, duration, fadeIn);
        }
        else {
          messageRow.css('opacity', 0)
            .show();
          fadeIn();
        }

        this.refreshDialogSizes();
      },

      hideMessage: function() {
        var messageRow = this.getMessageRow();

        if (messageRow.is(':visible')) {
          messageRow.stop(true)
            .animate({opacity: 0.2}, this.options.duration);
        }
      },

      errorMessage: function(message) {
        this.showMessage(message, 'ctx-error');
      },

      successMessage: function(message, close) {
        this.getEmailRow()
          .hide();
        this.showMessage(message, 'ctx-success');

        if (close) {
          setTimeout(this.proxy(this.close), 3000);
        }
      },

      broadcastPersonalizationStatus: function(status) {
        // Don't use the broadcast() method, as it's modified for broadcasting
        // the overlay state.
        var broadcastType = Contextly.widget.PersonalizationSubscribe.broadcastTypes.STATUS;
        Contextly.Transmitter.fn.broadcast.call(this, broadcastType, status);
      },

      refreshDialogSizes: function() {
        // Update max height of the stories section so that the dialog fits the
        // window.
        if (this.hasLinks()) {
          var dialogHeight = this.dialog.height();
          var sectionHeight = this.getStoriesSection().height();
          var windowHeight = this.window.height();
          var maxHeight = Math.floor(0.95 * windowHeight - (dialogHeight - sectionHeight));
          this.getStoriesSection().css('max-height', maxHeight);
        }

        return Contextly.overlay.Base.refreshDialogSizes.apply(this, arguments);
      }

    }

  });

})(jQuery);


