(function($) {

  /**
   * @class
   * @extends Contextly.overlay.Base
   */
  Contextly.overlay.StoryLineSubscribe = Contextly.createClass({

    extend: Contextly.overlay.Base,

    statics: /** @lends Contextly.overlay.StoryLineSubscribe */ {

      open: function(options) {
        Contextly.overlay.Base.open.call(this, null, options);
      },

      startOpening: function() {
        if (this.openingGate) {
          this.openingGate.destroy();
        }
        this.openingGate = new Contextly.CallbacksGate();
        this.openingReasons = {
          animation: this.openingGate.addReason(),
          request: this.openingGate.addReason()
        };

        Contextly.overlay.Base.startOpening.apply(this, arguments);
        this.registerButtonClick();

        this.openingGate.appendResult(this.proxy(this.unlockSubscriptionUI));
        this.openingGate.appendResult(this.proxy(this.focusEmailInput));
        this.openingGate.unseal();
      },

      completeOpening: function() {
        Contextly.overlay.Base.completeOpening.apply(this, arguments);

        // Resolve reason on opening complete.
        this.openingReasons.animation();
      },

      startClosing: function() {
        this.openingGate.destroy();

        Contextly.overlay.Base.startClosing.apply(this, arguments);
      },

      renderOverlay: function() {
        var init = Contextly.overlay.Base.renderOverlay.apply(this, arguments);
        if (init) {
          this.overlay.addClass('ctx-storyline-overlay');
        }
      },

      isNewsletterInputEnabled: function() {
        return this.options.subscribe_newsletter;
      },

      renderDialog: function() {
        var isNewsletterEnabled = this.isNewsletterInputEnabled();

        if (!this.dialog) {
          var classes = [ 'ctx-overlay-dialog' ];
          if (!Contextly.Browser.hasSvg()) {
            classes.push('ctx-no-svg');
          }

          var content =
            '<div id="ctx-storyline-popup" class="' + Contextly.Utils.escape(classes.join(' ')) + '">'
            + '<div class="ctx-header">'
            + '<a href="javascript:" class="ctx-dialog-close">X</a>'
            + '<span class="ctx-header-title">'
            + 'Follow this Storyline'
            + '</span>'
            + '</div>'
            + '<div class="ctx-body">'
            + '<div class="ctx-message-row"></div>'
            + '<div class="ctx-email-row ctx-input-row">'
            + '<input type="email" class="ctx-email" placeholder="Email address" />'
            + '</div>'
            + '<div class="ctx-newsletter-row ctx-input-row">'
            + '<input class="ctx-newsletter" type="checkbox" id="ctx-storyline-subscribe-newsletter" checked="checked">'
            + ' '
            + '<label class="ctx-newsletter-label" for="ctx-storyline-subscribe-newsletter">'
            + '</label>'
            + '</div>'
            + '<div class="ctx-follow-row ctx-input-row">'
            + '<button class="ctx-follow">'
            + '<div class="ctx-follow-label">Follow</div>'
            + '<div class="ctx-icon ctx-icon-hourglass"></div>'
            + '</button>'
            + '</div>'
            + '<div class="ctx-terms-row">'
            + "Don't worry. We <b>don't</b> spam, and <b>never</b> sell or rent addresses."
            + '</div>'
            + '</div>'
            + '<div class="ctx-footer ctx-clearfix">'
            + '<a class="ctx-how-link" target="_blank" href="http://contextly.com/follow-up-explained">How does this work?</a>'
            + '<a class="ctx-logo-link" target="_blank" href="http://contextly.com">Powered by</a>'
            + '</div>'
            + '</div>';
          this.dialog = $(content)
            .appendTo('body');
        }
        else {
          // Show dialog and reset its state after previous attempts.
          this.dialog
            .show();
          this.getMessage()
            .hide();
          this.getEmailInput()
            .val('');
          this.getInputRows()
            .show();
        }

        if (isNewsletterEnabled) {
          this.getNewsletterLabel()
            .text(this.options.subscribe_newsletter_title);
          this.getNewsletterRow()
            .show();

          // Avoid using jQuery.attr() as it works different in jQuery >=1.6
          // and <1.6. DOM property is more portable here.
          this.getNewsletterCheckbox()[0].checked = true;
        }
        else {
          this.getNewsletterRow()
            .hide();
        }

        // Subscription UI is locked initially while we're animating and
        // checking the session for confirmed e-mail.
        this.disableTransitions();
        this.lockSubscriptionUI();
        this.postponeTransitions();
      },

      disableTransitions: function() {
        this.dialog
          .removeClass('ctx-transitions');
      },

      postponeTransitions: function() {
        setTimeout(this.proxy(function() {
          this.dialog
            .not('.ctx-transitions')
            .addClass('ctx-transitions');
        }), 0);
      },

      bindHandlers: function() {
        Contextly.overlay.Base.bindHandlers.apply(this, arguments);

        this.getCloseLink()
          .bind(this.ns('click'), this.proxy(this.close));

        // It's still safe to bind UI handlers here, because we disable all
        // inputs on locked UI anyway.
        this.getSubmit()
          .bind(this.ns('click'), this.proxy(this.onSubmit, false, true));
        this.getEmailInput()
          .bind(this.ns('keypress'), this.proxy(this.onEmailInputKeyPress, false, true));
      },

      unbindHandlers: function() {
        Contextly.overlay.Base.unbindHandlers.apply(this, arguments);

        this.getCloseLink()
          .unbind(this.ns('click'));

        this.getSubmit()
          .unbind(this.ns('click'));
        this.getEmailInput()
          .unbind(this.ns('keypress'));
      },

      destroyDialog: function() {
        this.dialog.hide();
      },

      onEmailInputKeyPress: function(e) {
        if (e.which == 13) {
          this.onSubmit(e);
        }
      },

      onSubmit: function(e) {
        e.preventDefault();

        var params = {};
        var email = $.trim(this.getEmailInput().val());
        if (!email || !Contextly.Utils.isEmailValid(email)) {
          this.errorMessage('Please enter valid email address.');
          return;
        }

        params.email = email;
        if (this.isNewsletterInputEnabled()) {
          params.subscribe_newsletter = this.getNewsletterCheckbox().is(':checked') ? 1 : 0;
        }

        this.subscribeToStoryLine(params);
      },

      registerButtonClick: function() {
        var params = {
          auto_subscribe: true
        };
        var callback = this.proxy(this.onButtonClickRegistered, false, true);
        this.options.experienceClient.call('storylines', 'button-click', params, callback);
      },

      onButtonClickRegistered: function(response) {
        if (response.success && response.confirmed) {
          // This session has confirmed e-mail already, so we just show success
          // message and close the dialog without resolving the request reason.
          this.successMessage('We added this story to your FollowUp list.');
        }
        else {
          // If the request has failed or the session has no confirmed e-mail,
          // resolve the reason to unlock subscription UI.
          this.openingReasons.request();
        }
      },

      subscribeToStoryLine: function(params) {
        this.lockSubscriptionUI();
        this.hideMessage();

        params = params || {};
        var callback = this.proxy(this.onSubscribeComplete, false, true);
        this.options.experienceClient.call('storylines', 'subscribe', params, callback);
      },

      onSubscribeComplete: function(response) {
        if (response && response.success) {
          if (response.confirmed) {
            this.successMessage('We added this story to your FollowUp list.');
          }
          else {
            this.successMessage('Almost done! Check your email.');
          }
        }
        else {
          var message = 'Something went wrong.';
          if (response && response.error) {
            message = response.error;
          }
          this.errorMessage(message);
          this.unlockSubscriptionUI();
          this.focusEmailInput();
        }
      },

      lockSubscriptionUI: function() {
        this.getInputs()
          .attr('disabled', 'disabled');
        this.getInputRows()
          .addClass('ctx-progress');
        this.getSubmit()
          .attr('disabled', 'disabled')
          .not('.ctx-progress')
          .addClass('ctx-progress');
      },

      unlockSubscriptionUI: function() {
        this.getInputs()
          .removeAttr('disabled');
        this.getInputRows()
          .removeClass('ctx-progress');
        this.getSubmit()
          .removeAttr('disabled')
          .removeClass('ctx-progress');
      },

      focusEmailInput: function() {
        this.getEmailInput()
          .focus();
      },

      getMessage: function() {
        return this.dialog.find('.ctx-message-row');
      },

      getEmailInput: function() {
        return this.dialog.find('.ctx-email');
      },

      getNewsletterCheckbox: function() {
        return this.dialog.find('.ctx-newsletter');
      },

      getNewsletterLabel: function() {
        return this.dialog.find('.ctx-newsletter-label');
      },

      getNewsletterRow: function() {
        return this.dialog.find('.ctx-newsletter-row');
      },

      getSubmit: function() {
        return this.dialog.find('.ctx-follow');
      },

      getInputRows: function() {
        return this.dialog.find('.ctx-input-row');
      },

      getInputs: function() {
        var inputs = this.getEmailInput();

        if (this.isNewsletterInputEnabled()) {
          inputs = inputs.add(this.getNewsletterCheckbox());
        }

        return inputs;
      },

      getCloseLink: function() {
        return this.dialog.find('.ctx-dialog-close');
      },

      showMessage: function(text, className) {
        var message = this.getMessage();
        var duration = this.options.duration;

        var fadeIn = this.proxy(function() {
          message.removeClass('ctx-success ctx-error ctx-info')
            .addClass(className)
            .text(text)
            .animate({opacity: 1}, duration);
        });

        if (message.is(':visible')) {
          duration /= 2;
          message
            .stop(true)
            .animate({opacity: 0}, duration, fadeIn);
        }
        else {
          message.css('opacity', 0)
            .show();
          fadeIn();
        }

        this.refreshDialogSizes();
      },

      hideMessage: function() {
        var message = this.getMessage();

        if (message.is(':visible')) {
          message.stop(true)
            .animate({opacity: 0.2}, this.options.duration);
        }
      },

      errorMessage: function(message) {
        this.showMessage(message, 'ctx-error');
      },

      successMessage: function(message) {
        this.getInputRows()
          .hide();
        this.showMessage(message, 'ctx-success');
        setTimeout(this.proxy(this.close), 3000);
      }

    }

  });

})(jQuery);


