(function($) {

  Contextly.widget.ResponsiveWatcher = Contextly.createClass({

    extend: Contextly.ProxyEach.prototype,

    statics: {

      construct: function() {
        this.lastWidth = 0;
        this.resizeTimer = null;
        this.pollingTimer = null;
        this.pollingsLeft = {};
        this.binded = false;
        this.widgets = {};
        this.$window = $(window);
      },

      add: function(widget) {
        var index = widget.getWidgetIndex();
        if (this.widgets[index] != null) {
          return;
        }

        this.widgets[index] = widget;
        this.pollingsLeft[index] = 3;

        this.bind();
        this.startPolling();

        this.updateWidgetLayouts([widget]);
      },

      remove: function(widget) {
        var index = widget.getWidgetIndex();

        if (this.widgets[index] == null) {
          return;
        }
        delete this.widgets[index];

        if (this.pollingsLeft[index] != null) {
          delete this.pollingsLeft[index];
        }

        this.unbind();
      },

      ns: function(eventName) {
        return eventName + '.contextlyResponsiveLayout';
      },

      bind: function() {
        if (this.binded) {
          return;
        }

        this.$window
          .bind(this.ns('resize'), this.proxy(this.onResizeStart))
          .bind(this.ns('load'), this.proxy(this.onWindowLoad));
      },

      unbind: function() {
        if (!this.binded || !Contextly.Utils.isEmptyObject(this.widgets)) {
          return;
        }

        this.clearResizeTimer();
        this.$window
          .unbind(this.ns('resize'))
          .unbind(this.ns('load'));
      },

      clearResizeTimer: function() {
        if (this.resizeTimer != null) {
          clearTimeout(this.resizeTimer);
          this.resizeTimer = null;
        }
      },

      onResizeStart: function() {
        this.clearResizeTimer();
        this.resizeTimer = setTimeout(this.proxy(this.onResizeComplete), 300);
      },

      onResizeComplete: function() {
        this.resizeTimer = null;

        var currentWidth = this.$window.width();
        if (currentWidth != this.lastWidth) {
          this.updateWidgetLayouts();
        }
      },

      startPolling: function() {
        if (this.pollingTimer != null || Contextly.Utils.isEmptyObject(this.pollingsLeft)) {
          return;
        }

        this.pollingTimer = setTimeout(this.proxy(this.onPolling), 800);
      },

      onPolling: function() {
        var widgets = {};
        var pollingsLeft = {};
        this.each(this.pollingsLeft, function(left, index) {
          widgets[index] = this.widgets[index];

          if (left > 1) {
            pollingsLeft[index] = left - 1;
          }
        });
        this.pollingsLeft = pollingsLeft;
        if (!Contextly.Utils.isEmptyObject(widgets)) {
          this.updateWidgetLayouts(widgets);
        }

        this.pollingTimer = null;
        this.startPolling();
      },

      onWindowLoad: function() {
        this.pollingsLeft = {};
        this.updateWidgetLayouts();
      },

      updateWidgetLayouts: function(widgets) {
        widgets = widgets || this.widgets;
        this.each(widgets, function(widget) {
          widget.checkLayoutThresholds();
        });
      }

    }

  });

  /**
   * @mixin
   */
  Contextly.widget.ResponsiveLayout = Contextly.createClass(/** @lends Contextly.widget.ResponsiveLayout */{

    destroyReferences: function() {
      Contextly.widget.ResponsiveWatcher.remove(this);
    },

    getLayoutModes: function() {
      return {};
    },

    setUpResponsiveLayout: function() {
      if (Contextly.Utils.isEmptyObject(this.getLayoutModes())) {
        return;
      }

      Contextly.widget.ResponsiveWatcher.add(this);
    },

    sizeMatchesThresholds: function(size, thresholds) {
      return (size >= thresholds[0] && (!thresholds[1] || size < thresholds[1]));
    },

    checkLayoutThresholds: function() {
      var modes = this.getLayoutModes();
      var elements = this.getWidgetElements();
      this.eachElement(elements, function(element) {
        var width = element.width();
        this.each(modes, function(thresholds, name) {
          if (this.sizeMatchesThresholds(width, thresholds)) {
            this.setLayoutMode(element, name);
            return false;
          }
        });
      });
    },

    buildLayoutClass: function(mode) {
      return 'ctx-' + mode;
    },

    setLayoutMode: function(element, mode) {
      var key = 'ctxLayoutMode';
      var lastMode = element.data(key);

      if (mode === lastMode) {
        // Nothing to do.
        return;
      }

      // Be silent on the first time.
      var firstTime = (lastMode == null);

      if (!firstTime) {
        element.removeClass(this.buildLayoutClass(lastMode));
      }

      element.data(key, mode);
      element.addClass(this.buildLayoutClass(mode));

      if (!firstTime) {
        this.broadcast(Contextly.widget.broadcastTypes.LAYOUT_CHANGED, mode);
      }
    }

  });

})(jQuery);