(function($) {

  /**
   * Base class for all widgets displaying links list.
   *
   * @class
   * @extends Contextly.widget.Base
   * @extends Contextly.widget.ResponsiveLayout
   */
  Contextly.widget.BaseLinksList = Contextly.createClass( /** @lends Contextly.widget.BaseLinksList.prototype */ {

    extend: [
      Contextly.widget.Base,
      Contextly.widget.ResponsiveLayout
    ],

    destroyReferences: function() {
      this.getWidgetElements()
        .find("a[rel='ctx-video-dataurl']")
        .unbind('click');
      this.getWidgetContainers()
        .find(".ctx-branding-link")
        .unbind('click');

      Contextly.widget.ResponsiveLayout.fn.destroyReferences.apply(this, arguments);
      Contextly.widget.Base.fn.destroyReferences.apply(this, arguments);
    },

    construct: function(params) {
      Contextly.widget.Base.apply(this, arguments);

      this.widget_view_happened = false;
    },

    getScreenWidth: function () {
      return $(window).width();
    },

    hasWidgetData: function() {
      // Descendants must override it.
      return false;
    },

    getLinksLimit: function() {
      return 0;
    },

    isDisplaySection: function(section) {
      var display_section = $.inArray(section, this.widget.settings.display_sections) != -1;
      var have_to_display = this.widget.links && this.widget.links[ section ] && this.widget.links[ section ].length > 0;

      return display_section && have_to_display;
    },

    display: function() {
      var widgetHasData = this.hasWidgetData();

      if (widgetHasData) {
        var widget_html = this.getWidgetHTML();
        this.displayHTML( widget_html );
      }
      else {
        // Hide content of the snippet placeholders (e.g. Loading...)
        this.getWidgetContainers().empty();
      }

      this.attachHandlers(widgetHasData);
      this.broadcastWidgetDisplayed();
    },

    getHandlers: function(widgetHasData) {
      var handlers = Contextly.widget.Base.prototype.getHandlers.apply(this, arguments);

      if (widgetHasData) {
        handlers.setUpResponsiveLayout = true;
      }

      return handlers;
    },

    onVideoLinkClick: function(target, e) {
      e.preventDefault();

      var link = $(target);
      var data = {
        videoUrl: link.attr('href'),
        videoTitle: link.attr('title')
      };
      Contextly.overlay.Video.open(data);

      var contextly_url = link.attr('contextly-url');
      this.mainServerClient.call(contextly_url);
    },

    attachLinksPopups: function() {
      this.getWidgetElements()
        .find("a[rel='ctx-video-dataurl']")
        .bind('click', this.proxy(this.onVideoLinkClick, true, true));
    },

    getImageDimension: function() {
      return this.getImageDimensionFor(this.getSettings().images_type);
    },

    getImageDimensionFor: function(image_type) {
      image_type = image_type.replace('square', '').replace('letter', '');

      var dimensions = image_type.split('x');
      var w = 0;
      var h = 0;

      if (dimensions.length == 2) {
        w = parseInt(dimensions[0]);
        h = parseInt(dimensions[1]);
      }

      return {width: w, height: h};
    },

    getImagesHeight: function() {
      var image_dimension = this.getImageDimension();
      return image_dimension.height;
    },

    getImagesWidth: function() {
      var image_dimension = this.getImageDimension();
      return image_dimension.width;
    },

    getWidgetLinks: function() {
      if (this.getWidget() && this.getWidget().links) {
        return this.getWidget().links;
      }

      return null;
    },

    getModuleType: function() {
      return this.getSettings().display_type;
    },

    getWidgetSectionLinks: function(section) {
      var widget_links = this.getWidgetLinks();

      if (widget_links && widget_links[ section ]) {
        return widget_links[ section ];
      }

      return null;
    },

    getLinkThumbnailUrl: function(link) {
      if (link.thumbnail_url) {
        return link.thumbnail_url;
      }

      return null;
    },

    getLinkIconHTML: function(link) {
      if (link.video) {
        return '<span class="ctx-icon ctx-icon-video"></span>';
      } else if (link.tweet) {
        return '<span class="ctx-icon ctx-icon-twitter"></span>';
      }
      return '';
    },

    // TODO Check if any use cases left after moving to templates and drop.
    escape: function(text) {
      return Contextly.Utils.escape(text);
    },

    getEventTrackingHtml: function(link) {
      var onmousedown = '';

      if (!link.video) {
          onmousedown += "this.href=" + JSON.stringify(link.url) + ";"
      }

      var settings = this.getSettings();
      if (settings && settings.utm_enable) {
        onmousedown += this.getTrackLinkJS(link);
      }

      if (onmousedown === '') {
        return '';
      }
      return ' onmousedown="' + this.escape(onmousedown) + '"';
    },

    getLinkATag: function(link, content) {
      var attributes =
        this.renderAttributes({
          href: link.native_url,
          title: Contextly.Utils.htmlToText(link.title),
          "class": 'ctx-clearfix ctx-nodefs'
        })
        + this.getEventTrackingHtml(link);

      return "<a" + attributes + ">" + content + "</a>";
    },

    getVideoLinkATag: function(link, content) {
      var attributes =
        this.renderAttributes({
          rel: 'ctx-video-dataurl',
          "class": 'ctx-clearfix ctx-nodefs',
          href: link.native_url,
          title: Contextly.Utils.htmlToText(link.title),
          "contextly-url": link.url
        })
        + this.getEventTrackingHtml(link);

      return "<a" + attributes + ">" + content + "</a>";
    },

    getTweetLinkATag: function(link, content) {
      var attributes =
        this.renderAttributes({
          rel: 'ctx-tweet-dataurl',
          "class": 'ctx-clearfix ctx-nodefs',
          href: link.native_url,
          title: Contextly.Utils.htmlToText(link.title),
          "contextly-url": link.url
        })
        + this.getEventTrackingHtml(link);
      return "<a" + attributes + ">" + content + "</a>";
    },

    getTrackLinkJS: function(link) {
      var widget_type = JSON.stringify(this.getWidgetType());
      var link_type = JSON.stringify(link.type);
      var link_url = JSON.stringify(link.url);
      var link_title = JSON.stringify(link.title);

      return "Contextly.widget.PageEvents.trackLink(" + widget_type + "," + link_type + "," + link_url + "," + link_title + ");";
    },

    getLinkHTML: function(link, index) {
      if (link.video) {
        return this.getLinkHTMLVideo(link, index);
      } else if (link.tweet) {
        return this.getLinkHTMLTweet(link, index);
      } else {
        return this.getLinkHTMLNormal(link, index);
      }
    },

    getInnerLinkHTML: function(link) {
      // Link title contains HTML, API server takes care about it.
      return link.title;
    },

    getLinkHTMLVideo: function(link, index) {
      return "<div class='ctx-link ctx-" + this.escape(index) + "'>"
        + this.getVideoLinkATag(link, this.getInnerLinkHTML(link))
        + "</div>";
    },

    getLinkHTMLTweet: function(link, index) {
      return "<div class='ctx-link ctx-" + this.escape(index) + "'>"
        + this.getTweetLinkATag(link, this.getInnerLinkHTML(link))
        + "</div>";
    },

    getLinkHTMLNormal: function(link, index) {
      return "<div class='ctx-link ctx-" + this.escape(index) + "'>"
        + "<div class='ctx-link-title'>"
        + this.getLinkATag(link, this.getInnerLinkHTML(link))
        + "</div>"
        + "</div>";
    },

    isDisplayContextlyLogo: function() {
      return Contextly.Settings.isBrandingDisplayed();
    },

    getBrandingButtonHtml: function () {
      if (!this.isDisplayContextlyLogo()) {
        return '';
      }

      var classes = ['ctx-branding', 'ctx-clearfix'];
      if (!Contextly.Browser.hasSvg()) {
        classes.push('ctx-no-svg');
      }

      var div = "<div class='" + this.escape(classes.join(' ')) + "'>";
      div += "<a href='https://contextly.com' class='ctx-branding-link ctx-nodefs'>Powered by</a>";
      div += "</div>";
      return div;
    },

    attachBrandingHandlers: function() {
      if (!this.isDisplayContextlyLogo()) {
        return;
      }

      this.getWidgetContainers()
        .find(".ctx-branding-link")
        .bind('click', this.proxy(function(event) {
          event.preventDefault();
          Contextly.overlay.Branding.open();
        }, false, true));
    }

  });

})(jQuery);
