(function($) {

  /**
   * @class
   * @extends Contextly.widget.Base
   */
  Contextly.widget.ChannelSubscribe = Contextly.createClass( /** @lends Contextly.widget.ChannelSubscribe.prototype */ {

    extend: [Contextly.widget.Base, Contextly.Receiver],

    construct: function(params) {
      Contextly.widget.Base.apply(this, arguments);

      this.widget_type = Contextly.widget.types.CHANNEL_SUBSCRIBE;
    },

    destroyReferences: function() {
      this.detachChannelButtonHandler();
      this.detachChannelStatusUpdateHandler();

      Contextly.widget.Base.fn.destroyReferences.apply(this, arguments);
    },

    getWidgetLogoHTML: function() {
      var s = this.getSettings();

      var iconClasses = ['ctx-subscribe-logo'];
      if (s.css.logo === 'single-color') {
        iconClasses.push('ctx-icon', 'ctx-icon-logo');
      }
      else {
        iconClasses.push('ctx-image-logo');
      }
      return '<i class="' + this.escape(iconClasses.join(' ')) + '"></i>';
    },

    getWidgetLabelHTML: function() {
      var data = this.getWidget();
      var title = (this.getSettings().title + '')
        .replace('%channel%', data.channel_name)
        .replace('%site%', data.site_name);

      return '<span class="ctx-subscribe-label">'
        + this.escape(title)
        + '</span>';
    },

    getWidgetHTML: function() {
      var s = this.getSettings();

      var linkClasses = [
        'ctx-channel-link',
        'ctx-story-button',
        this.getWidgetStyleClass()
      ];
      linkClasses.push('ctx-height-' + (s.css.height || 'middle'));
      if (s.css.button_shadow) {
        linkClasses.push('ctx-box-shadow');
      }
      if (s.css.text_shadow) {
        linkClasses.push('ctx-text-shadow');
      }
      if (!Contextly.Browser.hasSvg()) {
        linkClasses.push('ctx-no-svg');
      }

      return '<a href="javascript:" class="' + this.escape(linkClasses.join(' ')) + '">'
        + '<span class="ctx-subscribe-row">'
        + this.getWidgetLogoHTML()
        + this.getWidgetLabelHTML()
        + '</span>'
        + '</a>';
    },

    attachChannelButtonHandler: function() {
      this.getWidgetContainers()
        .find('.ctx-channel-link')
        .bind('click', this.proxy(this.displayChannelPopup, false, true));
    },

    detachChannelButtonHandler: function() {
      this.getWidgetContainers()
        .find('.ctx-channel-link')
        .unbind('click');
    },

    attachChannelStatusUpdateHandler: function() {
      // Update stored status on changes made by other posts.
      var broadcastType = Contextly.widget.ChannelSubscribe.broadcastTypes.STATUS;
      this.receive(broadcastType, this.proxy(this.onChannelStatusChange, false, true));
    },

    detachChannelStatusUpdateHandler: function() {
      var broadcastType = Contextly.widget.ChannelSubscribe.broadcastTypes.STATUS;
      this.ignore(broadcastType, this.onChannelStatusChange);
    },

    displayChannelPopup: function (e) {
      e.preventDefault();

      var data = this.getWidget();
      Contextly.overlay.ChannelSubscribe.open({
        channel: data.channel,
        channel_name: data.channel_name,
        site_name: data.site_name,
        subscribe_newsletter: data.settings.subscribe_newsletter,
        subscribe_newsletter_title: data.settings.subscribe_newsletter_title,
        subscribed: data.subscribed,
        confirmed: data.confirmed,

        experienceClient: this.experienceClient
      });
    },

    getCustomCssCode: function() {
      var entry = {
        container: '.' + this.containers.getTypeClass(),
        widget: '.' + this.containers.getTypeClass() + ' .ctx-channel-link.' + this.getWidgetStyleClass()
      };
      return Contextly.widget.StorylineSubscribeCssCustomBuilder.buildCSS(entry, this.getSettings());
    },

    getWidgetStyleClass: function() {
      var theme = this.getSettings().theme || 'transparent';
      return 'ctx-theme-' + theme;
    },

    suppressTransitions: function() {
      var containers = this.getWidgetContainers();

      var selector;
      if (this.getSettings().theme === 'split-button') {
        selector = '.ctx-subscribe-label, .ctx-subscribe-logo';
      }
      else {
        selector = '.ctx-channel-link';
      }

      var suppressed = containers
        .find(selector)
        .addClass('ctx-no-transitions');
      setTimeout(function() {
        suppressed.removeClass('ctx-no-transitions');
      }, 1);
    },

    display: function() {
      var widget_html = this.getWidgetHTML();
      this.appendSpace();
      this.appendHTML(widget_html);
      this.appendSpace();
      this.suppressTransitions();
      this.attachHandlers();
      this.broadcastWidgetDisplayed();
    },

    appendSpace: function() {
      var textNode = document.createTextNode(' ');
      this.getWidgetContainers().append(textNode);
    },

    onChannelStatusChange: function(e, status) {
      // Ignore updates on different channels.
      if (!status.channel || this.getWidget().channel != status.channel) {
        return;
      }

      var keys = {subscribed: true, confirmed: true};
      for (var key in keys) {
        this.widget[key] = status[key];
      }
    },

    getHandlers: function() {
      var handlers = Contextly.widget.Base.prototype.getHandlers.apply(this, arguments);

      handlers.attachChannelButtonHandler = true;
      handlers.attachChannelStatusUpdateHandler = true;

      return handlers;
    }

  });

  Contextly.widget.ChannelSubscribe.broadcastTypes = {
    STATUS: 'contextlyChannelStatus'
  };

})(jQuery);
