(function($) {

  /**
   * @class
   * @extends Contextly.widget.Base
   * @extends Contextly.Receiver
   */
  Contextly.widget.PersonalizationSubscribe = Contextly.createClass( /** @lends Contextly.widget.PersonalizationSubscribe.prototype */ {

    extend: [Contextly.widget.Base, Contextly.Receiver],

    construct: function(params) {
      Contextly.widget.Base.apply(this, arguments);

      this.widget_type = Contextly.widget.types.PERSONALIZATION_SUBSCRIBE;
    },

    destroyReferences: function() {
      this.getWidgetContainers()
        .find('.ctx-personalization-link')
        .unbind('click');

      // TODO Unbind listener to the status updates also

      Contextly.widget.Base.fn.destroyReferences.apply(this, arguments);
    },

    getWidgetLogoHTML: function() {
      var s = this.getSettings();

      var iconClasses = ['ctx-subscribe-logo'];
      if (s.css.logo === 'single-color') {
        iconClasses.push('ctx-icon', 'ctx-icon-logo');
      }
      else {
        iconClasses.push('ctx-image-logo');
      }
      return '<i class="' + this.escape(iconClasses.join(' ')) + '"></i>';
    },

    getWidgetLabelHTML: function() {
      var title = '';
      var linksCount = this.getWidget().links.length;
      if (linksCount) {
        title = this.getSettings().title + ' (' + linksCount + ')';
      }
      else {
        title = this.getSettings().title_empty;
      }

      return '<span class="ctx-subscribe-label">'
        + this.escape(title)
        + '</span>'
    },

    getWidgetHTML: function() {
      var s = this.getSettings();

      var linkClasses = [
        'ctx-personalization-link',
        'ctx-story-button',
        this.getWidgetStyleClass()
      ];
      linkClasses.push('ctx-height-' + (s.css.height || 'middle'));
      if (s.css.button_shadow) {
        linkClasses.push('ctx-box-shadow');
      }
      if (s.css.text_shadow) {
        linkClasses.push('ctx-text-shadow');
      }
      if (!Contextly.Browser.hasSvg()) {
        linkClasses.push('ctx-no-svg');
      }

      return '<a href="javascript:" class="' + this.escape(linkClasses.join(' ')) + '">'
        + '<span class="ctx-subscribe-row">'
        + this.getWidgetLogoHTML()
        + this.getWidgetLabelHTML()
        + '</span>'
        + '</a>';
    },

    attachPersonalizationButtonHandler: function() {
      this.getWidgetContainers()
        .find('.ctx-personalization-link')
        .bind('click', this.proxy(this.displayPersonalizationPopup, false, true));
    },

    displayPersonalizationPopup: function (e) {
      e.preventDefault();

      var data = this.getWidget();
      Contextly.overlay.PersonalizationSubscribe.open({
        links: data.links,
        subscription: data.settings.subscription,
        subscribed: data.subscribed,
        confirmed: data.confirmed,

        experienceClient: this.experienceClient
      });
    },

    getCustomCssCode: function() {
      var entry = {
        container: '.' + this.containers.getTypeClass(),
        widget: '.' + this.containers.getTypeClass() + ' .ctx-personalization-link.' + this.getWidgetStyleClass()
      };
      return Contextly.widget.StorylineSubscribeCssCustomBuilder.buildCSS(entry, this.getSettings());
    },

    getWidgetStyleClass: function() {
      var theme = this.getSettings().theme || 'transparent';
      return 'ctx-theme-' + theme;
    },

    suppressTransitions: function() {
      var containers = this.getWidgetContainers();

      var selector;
      if (this.getSettings().theme === 'split-button') {
        selector = '.ctx-subscribe-label, .ctx-subscribe-logo';
      }
      else {
        selector = '.ctx-personalization-link';
      }

      var suppressed = containers
        .find(selector)
        .addClass('ctx-no-transitions');
      setTimeout(function() {
        suppressed.removeClass('ctx-no-transitions');
      }, 1);
    },

    display: function() {
      // We just make a request here, actual displaying is done on the request
      // received.
      var callback = this.proxy(this.onRecommendationsReceived, false, true);
      this.experienceClient.call('personalization', 'button', {}, callback);
    },

    onRecommendationsReceived: function(response) {
      if (!response.success || !response.entry) {
        return;
      }

      this.widget.links = response.entry.stories || [];
      if (!this.widget.links.length && !this.getSettings().subscription) {
        return;
      }

      this.widget.subscribed = response.entry.subscribed || false;
      this.widget.confirmed = response.entry.confirmed || false;

      this.displayButton();

      // Update stored status on changes in any overlay.
      var broadcastType = Contextly.widget.PersonalizationSubscribe.broadcastTypes.STATUS;
      this.receive(broadcastType, this.proxy(this.onPersonalizationStatusChange, false, true));
    },

    displayButton: function() {
      var widget_html = this.getWidgetHTML();
      this.appendSpace();
      this.appendHTML(widget_html);
      this.appendSpace();
      this.suppressTransitions();
      this.attachHandlers();
      this.broadcastWidgetDisplayed();
    },

    appendSpace: function() {
      var textNode = document.createTextNode(' ');
      this.getWidgetContainers().append(textNode);
    },

    onPersonalizationStatusChange: function(e, status) {
      var keys = {subscribed: true, confirmed: true};
      for (var key in keys) {
        this.widget[key] = status[key];
      }
    },

    getHandlers: function() {
      var handlers = Contextly.widget.Base.prototype.getHandlers.apply(this, arguments);

      handlers.attachPersonalizationButtonHandler = true;

      return handlers;
    }

  });

  Contextly.widget.PersonalizationSubscribe.broadcastTypes = {
    STATUS: 'contextlyPersonalizationStatus'
  };

})(jQuery);
