(function($) {

  /**
   * @class
   * @extends Contextly.widget.BaseFlatLinksList
   */
  Contextly.widget.Sidebar = Contextly.createClass( /** @lends Contextly.widget.Sidebar.prototype */ {

    extend: Contextly.widget.BaseFlatLinksList,

    construct: function(params) {
      Contextly.widget.BaseFlatLinksList.apply(this, arguments);

      this.widget_type = Contextly.widget.types.SIDEBAR;
    },

    getWidgetStyleClass: function() {
      return 'ctx-content-sidebar';
    },

    getWidgetHTML: function() {
      var html = '';

      var title = this.widget.name;
      if (title) {
        html += "<div class='ctx-sb-title'><p>" + this.escape(title) + "</p></div>";
      }

      var description = this.widget.description;
      if (description) {
        html += "<div class='ctx-sb-description'><p>" + this.escape(description) + "</p></div>";
      }

      html += "<div class='ctx-sb-content'>"
        + this.getLinksHTML()
        + "</div>";

      var classes = [
        'ctx-sidebar',
        this.getWidgetStyleClass(),
        'ctx-sidebar-' + this.widget.layout,
        'ctx-clearfix'
      ];
      return "<div class='" + this.escape(classes.join(' ')) + "'>" + html + "</div>";
    },

    getLinkATag: function(link, content) {
      var attributes =
        this.renderAttributes({
          href: link.native_url,
          title: Contextly.Utils.htmlToText(link.title),
          "class": 'ctx-clearfix ctx-nodefs ctx-no-images'
        })
        + this.getEventTrackingHtml(link);

      return "<a" + attributes + ">" + content + "</a>";
    },

    getLinkHTML: function(link) {
      var html = '';

      if (link.thumbnail_url) {
        var image_html = "<img src='" + this.escape(link.thumbnail_url) + "' />";
        var image_href;

        if (link.video) {
          image_href = this.getVideoLinkATag(link, image_html);
        }
        else {
          image_href = this.getLinkATag(link, image_html);
        }

        html += "<div class='ctx-sb-img'>" + image_href + "</div>";
      }

      // Link title is HTML and API serve takes care about it.
      var linkContent = link.title;
      if (!link.thumbnail_url) {
        linkContent = '<span class="ctx-icon ctx-bullet"></span>' + linkContent;
      }
      if (this.widget.settings.display_link_dates && link.publish_date) {
        linkContent +=
          "<br />"
          + "<span class='ctx-pub-date'>"
          + this.textualDateDiffHTML(link.publish_date)
          + "</span>";
      }

      var a;
      if (link.video) {
        a = this.getVideoLinkATag(link, linkContent);
      }
      else {
        a = this.getLinkATag(link, linkContent);
      }

      var classes = ['ctx-sb-text'];
      if (!link.thumbnail_url) {
        classes.push('ctx-sb-no-thumbnail');
      }
      html += "<div class='" + this.escape(classes.join(' ')) + "'><p>" + a + "</p></div>";

      html = "<div class='ctx-sb-link'><div class='ctx-sb-fotmater ctx-clearfix'>" + html + "</div></div>";
      return html;
    },

    textualDateDiffHTML: function(date) {
      if (date && date.length > 4) {
        var t = date.split(/[- :]/);
        var js_date = new Date(t[0], t[1] - 1, t[2], t[3], t[4], t[5]);

        var timestamp = js_date.getTime() / 1000;
        var difference = new Date().getTime() / 1000 - timestamp;

        var periods = new Array("sec", "min", "hour", "day", "week", "month", "year", "decade");
        var lengths = new Array("60", "60", "24", "7", "4.35", "12", "10");
        var ending;

        if (difference > 0) {
          // this was in the past
          ending = "ago";
        }
        else { // this was in the future
          return 'right now';
        }

        for (var j = 0; difference >= lengths[j]; j++) {
          difference /= lengths[j];
        }
        difference = Math.round(difference);

        if (difference != 1) {
          periods[j] += "s";
        }
        return difference + "&nbsp;" + periods[j] + "&nbsp;" + ending;
      }
    },

    getCustomCssCode: function() {
      return Contextly.widget.SidebarCssCustomBuilder
        .buildCSS('.' + this.containers.getTypeClass(), this.getSettings());
    },

    setUpResponsiveLayout: function() {
      if (this.widget.layout === 'wide') {
        // There is no sense in normal mode on wide layout as they look exactly
        // the same, just set mobile mode once.
        var elements = this.getWidgetElements();
        this.eachElement(elements, function(element) {
          this.setLayoutMode(element, 'mobile');
        });
      }
      else {
        Contextly.widget.BaseFlatLinksList.prototype.setUpResponsiveLayout.apply(this, arguments);
      }
    },

    getLayoutModes: function() {
      return {
        "mobile": [0, 400],
        "default": [400]
      };
    },

    checkLayoutThresholds: function() {
      var elements = this.getWidgetElements();

      // TODO Avoid hard-coding screen condition.
      if (this.getScreenWidth() <= 540) {
        this.eachElement(elements, function(element) {
          this.setLayoutMode(element, 'mobile');
        });
      }
      else {
        // Since the widget may be set to float left/right with no minimal width
        // we can't measure its size for adaptive layout. We use containers
        // instead (far from perfect, but should work for most cases).
        var modes = this.getLayoutModes();
        this.eachElement(elements, function(element) {
          var width = element.parent().width();
          this.each(modes, function(thresholds, name) {
            if (this.sizeMatchesThresholds(width, thresholds)) {
              this.setLayoutMode(element, name);
              return false;
            }
          });
        });
      }
    },

    buildLayoutClass: function(mode) {
      return 'ctx-sidebar-' + mode;
    }

  })

})(jQuery);
