(function($) {

  /**
   * @class
   * @extends Contextly.widget.BaseLinksList
   * @extends Contextly.widget.ViewHandler
   */
  Contextly.widget.TextSnippet = Contextly.createClass( /** @lends Contextly.widget.TextSnippet.prototype */ {

    extend: [
      Contextly.widget.BaseLinksList,
      Contextly.widget.ViewHandler
    ],

    construct: function(params) {
      Contextly.widget.BaseLinksList.apply(this, arguments);

      this.widget_type = Contextly.widget.types.SNIPPET;
    },

    destroyReferences: function() {
      Contextly.widget.ViewHandler.fn.destroyReferences.apply(this, arguments);
      Contextly.widget.BaseLinksList.fn.destroyReferences.apply(this, arguments);
    },

    getWidgetStyleClass: function() {
      return 'ctx-content-text';
    },

    getWidgetClasses: function() {
      var classes = [
        'ctx-module',
        'ctx-nodefs',
        this.getWidgetStyleClass()
      ];
      if (!Contextly.Browser.hasSvg()) {
        classes.push('ctx-no-svg');
      }
      return classes;
    },

    getWidgetHTML: function() {
      var div = "",
        i, section_name;

      // Get the list of visible sections first, so that all the section classes
      // work properly.
      var sections = [];
      for (i = 0; i < this.widget.settings.display_sections.length; i++) {
        section_name = this.widget.settings.display_sections[i];
        if (this.isDisplaySection(section_name)) {
          sections.push(section_name);
        }
      }

      div += "<div class='" + this.escape(this.getWidgetClasses().join(' ')) + "'>";
      div += "<div class='ctx-sections-container ctx-clearfix'>";

      for (i = 0; i < sections.length; i++ ) {
        section_name = sections[i];
        var section_key = section_name + '_subhead';
        var section_header = this.widget.settings[ section_key ];
        var section_classes = ['ctx-section', 'ctx-section-' + section_name];

        // Add zebra classes. Mark single column on the row with a special one.
        if (i % 2 === 0) {
          if (i + 1 === sections.length) {
            section_classes.push('ctx-wide');
          }
          else {
            section_classes.push('ctx-odd');
          }
        }
        else {
          section_classes.push('ctx-even');
        }

        div += "<div class='" + this.escape(section_classes.join(' ')) + "'>";

        if (section_header) {
          div += "<div class='ctx-links-header'><p class='ctx-nodefs'>" + this.escape(section_header) + "</p></div>";
        }

        div += "<div class='ctx-links-content'>" + this.getLinksHTMLOfType(section_name) + "</div>";
        div += "</div>";
      }
      div += "</div>";
      div += "</div>";

      div += this.getBrandingButtonHtml();

      return div;
    },

    hasWidgetData: function () {
      if (this.widget.links) {
        var links = this.widget.links;
        for (var prop in links) {
          if (links.hasOwnProperty(prop) && links[prop].length) {
            return true;
          }
        }
      }
      return false;
    },

    getLinksHTMLOfType: function(type) {
      var html = "";
      var widget = this.widget;

      if (widget.links && widget.links[ type ]) {
        var placeCounter = 0;
        for (var link_idx in widget.links[ type ]) {
          var link = widget.links[ type ][ link_idx ];

          if (link.id && link.title) {
            html += this.getLinkHTML(link, ++placeCounter);
          }
        }
      }

      return html;
    },

    getInnerLinkHTML: function(link) {
      var icon = this.getLinkIconHTML(link);
      if (icon === '') {
        icon = '<span class="ctx-icon ctx-bullet"></span>';
      }

      // Link title contains HTML, API server takes care about it.
      return icon + link.title;
    },

    getHandlers: function(widgetHasData) {
      var handlers = Contextly.widget.BaseLinksList.fn.getHandlers.apply(this, arguments);

      if (widgetHasData) {
        handlers.attachLinksPopups = true;
        handlers.attachBrandingHandlers = true;

        if (!Contextly.Settings.isAdmin()) {
          handlers.attachWidgetViewHandler = true;
        }
      }

      return handlers;
    },

    getCustomCssCode: function() {
      return Contextly.widget.TextCssCustomBuilder
        .buildCSS('.' + this.containers.getTypeClass(), this.getSettings());
    },

    buildLayoutClass: function(mode) {
      return 'ctx-module-' + mode;
    },

    getLayoutModes: function() {
      return {
        "mobile": [0, 450],
        "default": [450]
      }
    },

    findViewWatcherElement: function(widgetElement) {
      // Return the first link.
      return widgetElement.find('.ctx-section .ctx-link:first');
    },

    // We will override this method for log module display event
    broadcastWidgetDisplayed: function() {
      Contextly.widget.BaseLinksList.fn.broadcastWidgetDisplayed.apply(this, arguments);

      var apiSession = this.experienceClient.getApiSession();
      if (apiSession.hasId()) {
        this.logEvent(Contextly.widget.eventNames.MODULE_VIEW, {
          event_guid: apiSession.getId(),
          event_success: false
        });
      }

    }


  });

})(jQuery);
